# -*- coding: utf-8 -*-
'''
Author: David J. Morfe
Application Name: GeoLiberator
Functionality Purpose: Instill data quality upon address data
Phase: Beta
'''

name = "GeoLiberator"
from .geoliberator import GeoLiberator, parse_address, geoLiberate, autoGeoLiberate
